#!/bin/bash
#
#	MWF 23/03/10
#	
#	Script called by a UDEV rule to notify plug that a device has been added or removed
#
#	$1 can be one of a (add), d (delete)
#	$2 is the name of the device
#
#	For example, if a drive was attached to /dev/sda1 this script would be invoked as follows:
#
#		tvmmount.sh a sda1
#
#	Note. A variant of this file is stored in the tiavata project too (ConfigFiles/arm/Sheevaplug)
#
NEW_ACTION=$1
NEW_DEVICE=$2
ACTION_TIMESTAMP=`date`
MOUNT_DEVICE="y"
export NEW_ACTION NEW_DEVICE ACTION_TIMESTAMP

#
#	Device added?	
#
if [[ $NEW_ACTION == "a" ]]; then
	if [[ $MOUNT_DEVICE == "y" ]]; then
		mkdir -p /mnt/tvmobili/tvm$NEW_DEVICE
		mount /dev/$NEW_DEVICE /mnt/tvmobili/tvm$NEW_DEVICE
	fi
	
	#
	# Run any commands for installed apps
	#
	/usr/local/bin/tvmdevicenotify.sh $NEW_DEVICE "a" "tvmobili" $ID_VENDOR $ID_MODEL
fi

#
#	If we do NOT support mounting of the device automatically i.e. on desktop machines for instance
#	we will need to find where the device has been mounted. Since we know the device, it is simply
#	a matter of calling df and parsing the output to tie up the device to its mount point with
#	the filesystem
#


#
#	Device removed?	
#
if [[ $NEW_ACTION == "d" ]]; then
	if [[ $MOUNT_DEVICE == "y" ]]; then
		umount /mnt/tvmobili/tvm$NEW_DEVICE
		rmdir /mnt/tvmobili/tvm$NEW_DEVICE
	fi

	#
	# Run any commands for installed apps
	#
	/usr/local/bin/tvmdevicenotify.sh $NEW_DEVICE "d" $ID_VENDOR $ID_MODEL "tvmobili"
fi
