#!/bin/bash
#
#	Which platform are we packaging on Apple or Linux?
#
COMMAND=`echo $1 | tr "[:lower:]" "[:upper:]"` 
#CURRENT_USER=`who | grep console | cut -d " " -f 1`
CALLING_USER=`whoami`
ARTWORK_USER=$USER
NOSTARTONBOOTFILE=/Applications/TVMOBiLi.app/Contents/conf/nostartonboot.txt
STARTONBOOT="Y"

#
#	Try and determine if this is a Mac. Originally we used MACHTYPE for this but on Mac, when running a different shell i.e. tcsh
#	curiously the MACHTYPE var has less information in it and does not allow us to determine if the machine is a Mac. The other var
#	we can use is bash array env variable BASH_VERSINFO
#
TESTTYPEVAR=`echo $MACHTYPE | tr "[:lower:]" "[:upper:]"`
PLATFORM=`echo $TESTTYPEVAR | grep -i apple`
if [ "$PLATFORM" == "" ]; then
	TESTTYPEVAR=`echo ${BASH_VERSINFO[5]} | tr "[:lower:]" "[:upper:]"`
	PLATFORM=`echo $TESTTYPEVAR | grep -i apple`
fi

CopyPList()
{
	PLIST_DIR=$1
	PLIST_FILE=$2
	if [ ! -f $PLIST_DIR/$PLIST_FILE ]; then
		sudo cp /Applications/TVMOBiLi.app/Contents/MacOS/$PLIST_FILE $PLIST_DIR/$PLIST_FILE
		sudo chown root:wheel $PLIST_DIR/$PLIST_FILE
		sudo chmod 644 $PLIST_DIR/$PLIST_FILE
	fi
}

CheckRemovePList()
{
	PLIST_DIR=$1
	PLIST_FILE=$2

	if [ "$STARTONBOOT" == "N" ]; then
		sudo rm -f $PLIST_DIR/$PLIST_FILE
	fi			
}

UnloadService()
{
#	-w disables the daemon and prevents it from starting on machine bootup i.e. sudo lauchctl unload -w /Library/LaunchDaemons/com.tvmobili.tvmobilisvcd.com
#
	echo Stopping tvmobilisvcd...
	pushd /Library/LaunchDaemons > /dev/null
	CopyPList /Library/LaunchDaemons com.tvmobili.tvmobilisvcd.plist

	sudo launchctl unload com.tvmobili.tvmobilisvcd.plist

	CheckRemovePList /Library/LaunchDaemons com.tvmobili.tvmobilisvcd.plist
	popd > /dev/null
}

LoadService()
{
#	-w reenables the daemon i.e. sudo lauchctl load -w /Library/LaunchDaemons/com.tvmobili.tvmobilisvcd.com
#
	echo Starting tvmobilisvcd...
	pushd /Library/LaunchDaemons > /dev/null
	CopyPList /Library/LaunchDaemons com.tvmobili.tvmobilisvcd.plist

	sudo launchctl load -w -F com.tvmobili.tvmobilisvcd.plist

	CheckRemovePList /Library/LaunchDaemons com.tvmobili.tvmobilisvcd.plist
	popd > /dev/null
}

UnloadArtworkApp()
{
	echo Stopping artwork helper...
	pushd /Library/LaunchAgents > /dev/null
	CopyPList /Library/LaunchAgents com.tvmobili.artwork.plist

	if [[ $COMMAND == "NEWVERSION" ]]; then
		su $ARTWORK_USER -c 'launchctl unload com.tvmobili.artwork.plist'
	else
		launchctl unload com.tvmobili.artwork.plist
	fi

	CheckRemovePList /Library/LaunchAgents com.tvmobili.artwork.plist
	popd > /dev/null
}

LoadArtworkApp()
{
	echo Starting artwork helper...
	pushd /Library/LaunchAgents > /dev/null
	CopyPList /Library/LaunchAgents com.tvmobili.artwork.plist

	if [[ $COMMAND == "NEWVERSION" ]]; then
		su $ARTWORK_USER -c 'launchctl load -w -F com.tvmobili.artwork.plist'
	else
		launchctl load -w -F com.tvmobili.artwork.plist
	fi

	CheckRemovePList /Library/LaunchAgents com.tvmobili.artwork.plist
	popd > /dev/null
}

if [[ $PLATFORM == $TESTTYPEVAR ]]; then

	#
	#	Need to run as root otherwise the daemon will either fail to start or will be run
	#	under the wrong account
	#
	#	MWF 2/11/10
	#	Reversed this, could not restart the artwork handler by su'ing to the current user, got a broken socket error for some mad reason
	#	This script now must be called as the logged on user and the commands that need to be run as root are prefixed by sudo
	#
	#if [[ $CALLING_USER != "root" ]]; then
	#	echo
	#	echo "Must be run as root. Have you forgotten to prefix the command with sudo?"
	#	echo
	#	exit
	#fi
	if [ -f $NOSTARTONBOOTFILE ]; then
		STARTONBOOT="N"
	fi
	if [[ $COMMAND != "NEWVERSION" ]]; then
		if [[ $CALLING_USER == "root" ]]; then
			echo
			echo "Must be run as a regular user. You will be prompted for the su password when required."
			echo
			exit
		fi
	fi

	#
	#	Need to run the artwork handler in the account of the logged on user. This is easyish to determine but
	#	is done in different ways depending on how the script is being run. If the script is being run from
	#	the installer (most likely) then $USER should contain the name of the logged on user. However, if the
	#	script is called interactively via sudo then $USER will have changed to root but $SUDO_USER should
	#	contain the name of the logged on user.
	#
	#	As I understand it, it is a known issue with LaunchAgents that upgrading, installing is a nightmare
	#	without getting the user to log out or reboot. This is the preferred way since there is no messing
	#	around with user accounts and calling launchctl (which incidentally does not work properly - calling
	#	this script interactively to start the agents/daemons will not work for the artwork handler - you will
	#	get an error 'launch_msg(): Socket is not connected' and the agent will not start). No matter, this is
	#	the least likely use of this script and in any case the app will launch properly next time the user
	#	logs on. It is more important that this script behaves correctly during an upgrade / install.
	#
	ARTWORK_USER=$USER
	if [[ $ARTWORK_USER == "root" ]]; then
		ARTWORK_USER=$SUDO_USER
	fi

	if [[ $COMMAND == "STOP" ]]; then
	
		UnloadService
		UnloadArtworkApp

	elif [[ $COMMAND == "START" ]]; then

		LoadService
		LoadArtworkApp

	elif [[ $COMMAND == "RESTART" ]]; then

		UnloadArtworkApp
		UnloadService
		sleep 2
		LoadService
		LoadArtworkApp

	elif [[ $COMMAND == "AUTOSTART" ]]; then

		#
		#	Remove the marker file to indicate TVMOBiLi should not start on boot
		#
		if [ -f $NOSTARTONBOOTFILE ]; then
			rm -f $NOSTARTONBOOTFILE
		fi 
		STARTONBOOT="Y"
		CopyPList /Library/LaunchDaemons com.tvmobili.tvmobilisvcd.plist
		CopyPList /Library/LaunchAgents com.tvmobili.artwork.plist

	elif [[ $COMMAND == "NOAUTOSTART" ]]; then

		#
		#	Add the marker file to indicate TVMOBiLi should not start on boot
		#
		touch $NOSTARTONBOOTFILE
		STARTONBOOT="N"
		CheckRemovePList /Library/LaunchDaemons com.tvmobili.tvmobilisvcd.plist
		CheckRemovePList /Library/LaunchAgents com.tvmobili.artwork.plist

	elif [[ $COMMAND == "NEWVERSION" ]]; then
		UnloadArtworkApp
		UnloadService

		rm /Applications/TVMOBiLi.app/Contents/MacOS/com.tvmobili.tvmobilisvcd.plist.root
		rm /Applications/TVMOBiLi.app/Contents/MacOS/com.tvmobili.tvmobilisvcd.plist.tvmobili
		
		#
		#	Just in case, mop up any stragglers. It does appear that we have 1 too many qlmanager processes running but at this point
		#	however we do not know why. Unfortunately - the extra process is the 'right' process and is not controllable via launchagent
		# 	hence on a reinstall, we will need to kill it to pick up the new binary.
		#	The 'too many qlmanagers' issue is possibly due to a bug in StartProcess which possibly resulted in 2 processes being started
		#	(just because of the way it works). StartProcess launches a process to launch the actual required process but the first process
		#	does not terminate. This is just a theory at the moment but thought I'd document it here in case this gets investigated again.
		#
		#	Not sure this is the case anymore, think this was just a failure in the way we were starting the qlmanager i.e. we were starting
		#	it in the context of the root account during an upgrade which spawned the second instance of the process.
		#
		#ps -A | grep -i tvm | awk '$5 ~ /\/Applications\/TVMOBiLi.app\/Contents\/MacOS\/tvmobiliqlhandler/ { print "kill -9",$1;}' > /tvmql--XX.sh
		#chmod 777 /tvmql--XX.sh
		#/tvmql--XX.sh
		#rm -f tvmql--XX.sh
		sleep 2
		LoadService
		LoadArtworkApp

	else

		echo ""
		echo "Usage: ctltvm.sh [start | stop | restart | autostart | noautostart ] "
		echo ""
	fi

else
	
	/etc/init.d/tvmobilisvcd $COMMAND

fi
