/**
 * global variables
 */
var currentMedia = null;
var currentVolume = 0.5;
var progressFlag = 1;
var session = null;
var currentMediaURL = "";
var currentMediaMimeType = "";
var isInitialised = false;

//
// function that sets the initialised state of the api
//
function setCastInitialised(init)
{
    logMessage("setCastInitialised()");
    isInitialised = init;
}

/**
 * initialization
 */
function initializeCastApi(onInitSuccess, onInitError) 
{
    logMessage("initializeCastApi()");

    // make sure the app is not already initialised
	if (isInitialised)
	{
		onInitSuccess();
		return;
	}

	// default app ID to the default media receiver app
	// optional: you may change it to your own app ID/receiver
	var applicationID = "BC20331D"; //chrome.cast.media.DEFAULT_MEDIA_RECEIVER_APP_ID;
	var sessionRequest = new chrome.cast.SessionRequest(applicationID);
	var apiConfig = new chrome.cast.ApiConfig(sessionRequest, sessionListener, receiverListener);

	// now initialise chromecast
	chrome.cast.initialize(apiConfig, onInitSuccess, onInitError);
};

/**
 * generic success callback
 */
function onSuccess(message)
{
    logMessage("onSuccess" + message);
}

function onError()
{
    logMessage("onError");
}

/**
 * callback on success for stopping app
 */
function onStopAppSuccess()
{
    logMessage('Session stopped');
    session = null;
}

/**
 * session listener during initialization
 */
function sessionListener(e)
{
    logMessage('sessionListener() - ID: ' + e.sessionId);
    session = e;

    //
    // check to see if there is any media already playing
    //
    if (session.media.length != 0)
    {
        logMessage('Found ' + session.media.length + ' existing media sessions.');
        onMediaDiscovered('sessionListener', session.media[0]);
    }

    //
    // setup the listeners for the session and the media feedback
    //
    session.addMediaListener(onMediaDiscovered.bind(this, 'addMediaListener'));
    session.addUpdateListener(sessionUpdateListener.bind(this));  
}

/**
 * session update listener 
 */
function sessionUpdateListener(isAlive)
{
  var message = isAlive ? 'Session Updated' : 'Session Removed';
  message += ': ' + session.sessionId;
  logMessage(message);

    if (!isAlive)
    {
        //
        // session has ended
        //
        setResumeState(false);
    }
    else
    {
        setResumeState(true);
    }
};

/**
 * receiver listener during initialization
 */
function receiverListener(e)
{
    if (e === 'available')
    {
        logMessage("receiver found");
    }
    else
    {
        logMessage("receiver list empty");
    }
}

/**
 * select a media URL 
 * @param {string} m An index for media URL
 */
function selectVideoMedia(id, mime)
{
    var title = document.getElementById("description-" + id).innerText;
    var image = document.getElementById("artwork-" + id).innerText;
    var uri = document.getElementById("uri-" + id).innerText;

    logMessage("selectVideoMedia: " + uri + ", Mime: " + mime);
	currentMediaURL = uri; 
	currentMediaMimeType = mime;

    //
    // initialise the meta data object
    //
	var meta = new chrome.cast.media.MovieMediaMetadata();

	if (title) {
	    meta.title = title;
	}

	if (image) {
	    var im = new chrome.cast.Image(image);
	    meta.images = new Array(im);
	}

	$("#chromecast-controls").animate({ left: "-500px" });

	var msg = document.getElementById("chromecast-info-message");
	msg.innerHTML = "BUFFERING...";

	var head = document.getElementById("chromecast-info-header");
	head.innerHTML = "OK, here goes...";

    //
    // load the media
    //
	loadMedia(meta);
}

//
// helper function for displaying 
//
function selectImageMedia(id, mime)
{
    var title = document.getElementById("description-" + id).innerText;
    var image = document.getElementById("artwork-" + id).innerText;
    var uri = document.getElementById("uri-" + id).innerText;

    //
    // initialise the meta data object
    //
    var meta = new chrome.cast.media.PhotoMediaMetadata();
    if (title) {
        meta.title = title;
    }

    if (image) {
        var im = new chrome.cast.Image(image);
        meta.images = new Array(im);
    }

    logMessage("selectImageMedia: " + uri + ", Mime: " + mime);
    currentMediaURL = uri;
    currentMediaMimeType = mime;

    $("#chromecast-controls").animate({ left: "-500px" });

    var msg = document.getElementById("chromecast-info-message");
    msg.innerHTML = "BUFFERING...";

    var head = document.getElementById("chromecast-info-header");
    head.innerHTML = "OK, here goes...";

    //
    // load the media
    //
    loadMedia(meta);
}

//
// function that selects audio media to playback
//
function selectAudioMedia(id, mime)
{
    var title = document.getElementById("description-" + id).innerText;
    var artist = document.getElementById("artist-" + id).innerText;
    var album = document.getElementById("album-" + id).innerText;
    var image = document.getElementById("artwork-" + id).innerText;
    var uri = document.getElementById("uri-" + id).innerText;

    //
    // initialise the meta data object
    //
    var meta = new chrome.cast.media.MusicTrackMediaMetadata();

    if (artist)
    {
        meta.albumArtist = artist;
        meta.artist = artist;
        meta.subtitle = artist;
    }

    if (title)
    {
        meta.title = title;
    }

    if ( album)
    {
        meta.album = album;
    }

    if ( image )
    {
        var im = new chrome.cast.Image(image);
        meta.images = new Array(im);
    }

    logMessage("selectAudioMedia: " + uri + ", Mime: " + mime);
    currentMediaURL = uri;
    currentMediaMimeType = mime;

    $("#chromecast-controls").animate({ left: "-500px" });

    var msg = document.getElementById("chromecast-info-message");
    msg.innerHTML = "BUFFERING...";

    var head = document.getElementById("chromecast-info-header");
    head.innerHTML = "OK, here goes...";

    //
    // load the media
    //
    loadMedia(meta);
}

/**
 * launch app and request session
 */
function launchApp(onRequestSessionSuccess) 
{
	logMessage("launching app...");
	chrome.cast.requestSession(onRequestSessionSuccess, onLaunchError);
}

//
// set the enabled state
//
function setCastButtonEnabled(e)
{
    var btn = document.getElementById("chromecast-info-button");
    var msg = document.getElementById("chromecast-info-message");
    if (e)
    {
        btn.disabled = false;
        msg.innerText = "Press the 'cast' button to start your movie...";
        btn.className = "general-submit";
    }
    else
    {
        btn.disabled = true;
        msg.innerText = "Waiting for device selection...";
        btn.className = "general-submit-disabled";
    }
}

/**
 * callback on success for requestSession call  
 * @param {Object} e A non-null new session.
 */
function handleRequestSessionSuccess(e) 
{
	logMessage("session success: " + e.sessionId);

	//
	// update the button so that it is enabled
    //
	setCastButtonEnabled(true);

	session = e;
	session.addUpdateListener(sessionUpdateListener.bind(this));  
	if (session.media.length != 0) 
	{
		onMediaDiscovered('handleRequestSessionSuccess', session.media[0]);
	}

	session.addMediaListener(onMediaDiscovered.bind(this, 'addMediaListener'));
	session.addUpdateListener(sessionUpdateListener.bind(this));  
}

/**
 * callback on launch error
 */
function onLaunchError(err)
{
    logMessage("launch error" + err.toString());
}

/**
 * stop app/session
 */
function stopApp()
{
    logMessage("stopApp");
    if (session )
        session.stop(onStopAppSuccess, onError);
}

/**
 * load media
 * @param {string} i An index for media
 */
function loadMedia(meta) 
{
    logMessage("loadMedia");

    if (!session)
	{
		logMessage("no session");
		return;
	}

	logMessage("loading..." + currentMediaURL + " (" + currentMediaMimeType + ")");

	var mediaInfo = new chrome.cast.media.MediaInfo(currentMediaURL);
	mediaInfo.contentType = currentMediaMimeType;

    //
    // initialise the meta data
    //
    if ( meta )
	    mediaInfo.metadata = meta;

	var request = new chrome.cast.media.LoadRequest(mediaInfo);
	request.autoplay = true;
	request.currentTime = 0;
  
    //
    // physically call the load media method
    //
	session.loadMedia(request, onMediaDiscovered.bind(this, 'loadMedia'), onMediaError);
}

/**
 * callback on success for loading media
 * @param {Object} e A non-null media object
 */
function onMediaDiscovered(how, media) 
{
    logMessage("onMediaDiscovered:" + media.mediaSessionId + ' (' + how + ') [' + media.playerState + ']');

	currentMedia = media;
	currentMedia.addUpdateListener(onMediaStatusUpdate);
}

//
// set the resume state
//
function setResumeState(pause)
{
    var state = pause ? "PAUSE" : "RESUME";
    logMessage("setResumeState: " + state);
    var playpauseresume = document.getElementById("playpauseresume");
    if (!pause)
    {
        playpauseresume.src = '/images/chrome/pause.png';
    }
    else
    {
        playpauseresume.src = '/images/chrome/play.png';
    }

}

/**
 * callback on media loading error
 * @param {Object} e A non-null media object
 */
function onMediaError(e) 
{
  logMessage("media error");
}

/**
 * callback for media status event
 * @param {Object} e A non-null media object
 */
function onMediaStatusUpdate(isAlive) 
{
    var state = currentMedia ? currentMedia.playerState : "N/A";
    logMessage("onMediaStatusUpdate: " + state);

    //
    // work out the current media state
    //
    if (currentMedia)
    {
        switch (currentMedia.playerState)
        {
            case chrome.cast.media.PlayerState.IDLE:
            case chrome.cast.media.PlayerState.PAUSED:

                //
                // show the resume button
                //
                setResumeState(true);
                break;

            default:

                //
                // show the pause button
                //
                var head = document.getElementById("chromecast-info-header");
                head.innerHTML = "Go watch your cast.";
                setResumeState(false);
                break;
        }
    }

    if( progressFlag ) 
    {
        //document.getElementById("progress").value = parseInt(100 * currentMedia.currentTime / currentMedia.media.duration);
        //document.getElementById("progress_tick").innerHTML = currentMedia.currentTime;
        //document.getElementById("duration").innerHTML = currentMedia.media.duration;
    }

    document.getElementById("chromecast-info-message").innerHTML = currentMedia.playerState;
}

/**
 * Updates the progress bar shown for each media item.
 */
function updateCurrentTime() 
{
    logMessage("updateCurrentTime()");

    if (!session || !currentMedia)
    {
        return;
    }

    if (currentMedia.media && currentMedia.media.duration != null) 
    {
        var cTime = currentMedia.getEstimatedTime();
        document.getElementById("progress").value = parseInt(100 * cTime / currentMedia.media.duration);
        document.getElementById("progress_tick").innerHTML = cTime;
    }
    else 
    {
        document.getElementById("progress").value = 0;
        document.getElementById("progress_tick").innerHTML = 0;
    }
};

/**
 * play media
 */
function playMedia() 
{
    logMessage("playMedia()");
    if (!currentMedia)
        return;

    var playpauseresume = document.getElementById("playpauseresume");
    switch (currentMedia.playerState)
    {
        case chrome.cast.media.PlayerState.IDLE:
        case chrome.cast.media.PlayerState.PAUSED:

            //
            // show the resume button
            //
            currentMedia.play(null, mediaCommandSuccessCallback.bind(this, "resumed " + currentMedia.sessionId), onError);
            playpauseresume.src = '/images/chrome/pause.png';
            logMessage("playMedia: resumed");
            break;

        default:

            //
            // show the pause button
            //
            currentMedia.pause(null, mediaCommandSuccessCallback.bind(this, "paused " + currentMedia.sessionId), onError);
            playpauseresume.src = '/images/chrome/play.png';
            logMessage("playMedia: paused");
            break;
    }
}

/**
 * stop media
 */
function stopMedia() 
{
    logMessage("stopMedia()");
    if (!currentMedia)
        return;

    currentMedia.stop(null, mediaCommandSuccessCallback.bind(this, "stopped " + currentMedia.sessionId), onError);
    var playpauseresume = document.getElementById("playpauseresume");
    playpauseresume.src = '/images/chrome/play.png';
    logMessage("stopMedia: media stopped");
}

/**
 * set receiver volume
 * @param {Number} level A number for volume level
 * @param {Boolean} mute A true/false for mute/unmute 
 */
function setReceiverVolume(level, mute) 
{
    logMessage("setReceiverVolume(): " + level.toString());
    if (!session)
		return;

	if( !mute ) 
	{
		session.setReceiverVolumeLevel(level, mediaCommandSuccessCallback.bind(this, 'media set-volume done'), onError);
		currentVolume = level;
	}
	else 
	{
		session.setReceiverMuted(true, mediaCommandSuccessCallback.bind(this, 'media set-volume done'), onError);
	}
}

/**
 * mute media
 * @param {DOM Object} cb A checkbox element
 */
function muteMedia(cb) 
{
    logMessage("muteMedia()");
    if (cb.checked == true)
	{
		document.getElementById('muteText').innerHTML = 'Unmute media';
		setReceiverVolume(currentVolume, true);
		logMessage("media muted");
	}
	else 
	{
		document.getElementById('muteText').innerHTML = 'Mute media';
		setReceiverVolume(currentVolume, false);
		logMessage("media unmuted");
	} 
}

/**
 * seek media position
 * @param {Number} pos A number to indicate percent
 */
function seekMedia(pos) 
{
	logMessage('Seeking ' + currentMedia.sessionId + ':' + currentMedia.mediaSessionId + ' to ' + pos + "%");
	progressFlag = 0;

	var request = new chrome.cast.media.SeekRequest();
	request.currentTime = pos * currentMedia.media.duration / 100;
	currentMedia.seek(request, onSeekSuccess.bind(this, 'media seek done'), onError);
}

/**
 * callback on success for media commands
 * @param {string} info A message string
 * @param {Object} e A non-null media object
 */
function onSeekSuccess(info) 
{
	logMessage("onSeekSuccess: " + info);
	setTimeout(function(){progressFlag = 1},1500);
}

/**
 * callback on success for media commands
 * @param {string} info A message string
 * @param {Object} e A non-null media object
 */
function mediaCommandSuccessCallback(info) 
{
    logMessage("mediaCommandSuccessCallback: " + info);
}


//
// utility function that logs messages out into the console and a div on the current form if required
//
function logMessage(message)
{
	console.log(message);
	var dw = document.getElementById("debugmessage");
	if ( dw )
		dw.innerHTML += '<br />' + JSON.stringify(message);
}

