var tvm_websocket = null;
var tvm_websocket_timer = 0;
var wsUri = "ws://" + location.host + "/";

//
// function that logs out messages
//
void logMessage(msg)
{
    console.log(msg);
}

//
// function that handles status messages from the web socket
//
function processStatusMessage(msg) {

    //
    // the message will be split into parts using the '@' char
    //
    var status_data = msg.split('@');
    if (status_data) {

        //
        // each status is separated by the ':' char
        //
        var cat = status_data[0].split(':');
        if (cat) {
            if (cat[1] == '1') {
                logMessage('status-cataloguing');
                $('#cataloguing-div').attr('class', 'top-bar-div status-cataloguing');
                $('#cataloguing-div').attr('title', 'Cataloguing');
            }
            else {
                logMessage('status-not-cataloguing');
                $('#cataloguing-div').attr('class', 'top-bar-div status-not-cataloguing');
                $('#cataloguing-div').attr('title', 'Idle');
            }
        }

        //
        // next chunk is the alert status
        //
        var alertStatus = status_data[1].split(':');
        if (alertStatus) {
            if (alertStatus[1] == '1') {
                logMessage('status-alerting');
                $('#system-status-div').attr('class', 'top-bar-div status-alerting');
                $('#system-status-div').attr('title', 'Alerts Pending');
            }
            else {
                logMessage('status-not-alerting');
                $('#system-status-div').attr('class', 'top-bar-div status-not-alerting');
                $('#system-status-div').attr('title', 'System Healthy');
            } 
        }

        //
        // next chunk is the transcode status
        //
        var transStatus = status_data[2].split(':');
        if (transStatus) {
            if (transStatus[1] == '1') {
                logMessage('status-transcoding');
                $('#transcode-status-div').attr('class', 'top-bar-div status-transcoding');
                $('#transcode-status-div').attr('title', 'Transcoding');
            }
            else {
                logMessage('status-not-transcoding');
                $('#transcode-status-div').attr('class', 'top-bar-div status-not-transcoding');
                $('#transcode-status-div').attr('title', 'Not Transcoding');
            }
        }

        //
        // now it is the purchase status
        //
        var purchaseStatus = status_data[3].split(':');
        if (purchaseStatus) {
            if (purchaseStatus[1] == '1') {
                logMessage('status-purchased');
                $('#purchase-status-div').attr('class', 'top-bar-div status-purchased');
                $('#purchase-status-div').attr('title', 'Licenced');
                $('#purchase-status-div').html('<font color=\"#ffffff\">&nbsp;LICENCED</font>');
            }
            else {
                logMessage('status-not-purchased');
                $('#purchase-status-div').attr('class', 'top-bar-div status-not-purchased');
                $('#purchase-status-div').attr('title', 'Click to purchase');
                $('#purchase-status-div').html('<a href=\"#\" style=\"color:#ffffff\" onclick=\"purchaseTVMOBiLi();\">&nbsp;PURCHASE</a>');
            }
        }

    }
}

//
// function that handles websocket timeouts
//
function handleTVMOBiLiWebSocketTimeout() {

    try {
        //
        // create the web socket
        //
        if (tvm_websocket == null) {
            console.log('handleTVMOBiLiWebSocketTimeout | initialiseTVMOBiLiWebSockets()');
            initialiseTVMOBiLiWebSockets();
        }
        else {
            console.log('handleTVMOBiLiWebSocketTimeout | nothing to do...' + tvm_websocket.readyState);
        }
    }
    catch (e) {

        //
        // failed...
        //
        tvm_websocket = null;
        console.log('handleTVMOBiLiWebSocketTimeout | ' + e.toString());
    }
}

//
// function that initialises the websockets
//
function initialiseTVMOBiLiWebSockets() {

    //
    // just call the timeout handler immediately
    //
    if ( tvm_websocket == null )
        tvm_websocket = new WebSocket(wsUri, 'tvmobili');

    if ( tvm_websocket_timer == 0 )
        tvm_websocket_timer = setInterval("handleTVMOBiLiWebSocketTimeout()", 10000);

    //Connected to server
    tvm_websocket.onopen = function (ev) {
        console.log('tvm_websocket.onopen');        
    };

    //Connection close
    tvm_websocket.onclose = function (ev) {
        console.log('tvm_websocket.onclose');
        tvm_websocket = null;
    };

    //Message Receved
    tvm_websocket.onmessage = function (ev) {
        console.log('tvm_websocket.onmessage: ' + ev.data);

        //
        // determine what to do with the data.
        //
        var msg_data = ev.data.split("-");
        if (msg_data) {
            if (msg_data[0] == 'status') {
                processStatusMessage(msg_data[1]);
            }
        }
    };

    //Error
    tvm_websocket.onerror = function (ev) {
        console.log('tvm_websocket.onerror');
        tvm_websocket = null;
    };
}